{*******************************************************}
{                                                       }
{       SoftReal TAPI Components demo                   }
{                                                       }
{       Copyright (c) 2002-2004 SoftReal Kft.           }
{       All rights reserved.                            }
{                                                       }
{       Version:      V1.1 R2                           }
{       Compiler:     Delphi 5                          }
{       System:       Windows 98, Me, NT4, 2K, XP       }
{       Programmers:  2002 L.G.Horvath                  }
{                     2004 L.G.Horvath                  }
{       E-mail:       softreal@softreal.hu              }
{                                                       }
{*******************************************************}

unit DispCall;

{ The form defined in this unit will show call state changes during
  and outgoing or incoming dial-up call. }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls;

type
  TDispCallForm = class(TForm)
    Image1: TImage;
    AddressLabel: TLabel;
    CallStateLabel: TLabel;
    CancelButton: TButton;
    AutoCloseCheckBox: TCheckBox;
    procedure CancelButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    function DoStartCall(const ACaption, AnAddress: string): string;
  public
    function StartCall(const AnAddress: string): string; overload;
      { Start of an outgoing call.
        Fills the labels. The result is the content of AddressLabel.
        ShowModal should be called for showing the form, mrAbort reports
        user abort. }
    function StartCall(const CallerID: string; Flag: Cardinal;
        Starting: Boolean): string; overload;
      { Start of an incoming call, or CallerID has changed.
        Fills the labels. The result is the content of AddressLabel.
        ShowModal should be called for showing the form, mrAbort reports
        user abort. }
    function CallState(State, Info, Priv: Cardinal): string;
      { Fills the CallStateLabel and also returns it's content. }
    function EndCall: string;
      { Sets ModalResult to mrOk, result is 'Call finished.'. }
  end;

var
  DispCallForm: TDispCallForm;

implementation

{$R *.DFM}

uses Tapi, SrTapi;

{ TDispCallForm }

function TDispCallForm.DoStartCall(const ACaption,
  AnAddress: string): string;
begin
  Result:= AnAddress;
  Caption:= ACaption;;
  AddressLabel.Caption:= Result;
  CallStateLabel.Caption:= '';
  CancelButton.Caption:= 'Cancel';
  CancelButton.Enabled:= False;
end;

function TDispCallForm.StartCall(const AnAddress: string): string;
begin
  Result:= DoStartCall('Dialing', 'Calling ' + AnAddress);
end;

function TDispCallForm.StartCall(const CallerID: string; Flag: Cardinal;
  Starting: Boolean): string;
var
  S: string;
begin
  S:= 'Receiving call: ';
  if CallerID <> '' then S:= S + CallerID
  else S:= S + tapiCallPartyIDFlagToStr(Flag);
  if Starting then
    Result:= DoStartCall('Answering', S)
  else begin
    AddressLabel.Caption:= S;
    Result:= S;
  end;
end;

function TDispCallForm.CallState(State, Info, Priv: Cardinal): string;
begin
  Result:= tapiCallStateToStr(State, Info, Priv);
  CallStateLabel.Caption:= Result;
  CancelButton.Enabled:= True;
  if State = LINECALLSTATE_CONNECTED then begin
    CancelButton.Caption:= 'Close';
    if AutoCloseCheckBox.Checked then Close;
  end;
end;

function TDispCallForm.EndCall: string;
begin
  Result:= 'Call finished.';
  // a Windows message has to be generated here to exit the ShowModal loop
  // changing TLabel.Caption is suitable for this purpose
  CallStateLabel.Caption:= Result;
  ModalResult:= mrOk;
end;

procedure TDispCallForm.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TDispCallForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if (Action = caHide) and (ModalResult <> mrOk) then begin
    if CancelButton.Caption = 'Cancel' then
      ModalResult:= mrAbort
    else
      ModalResult:= mrCancel;
  end;
end;

end.
